//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "stdinclude.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)


Board::Board()
{
    m_ci=GetCardsInterface();
    m_ci->Init();
}

Board::~Board()
{
    m_ci->Term();
    delete m_ci;
    Clear();
    Move=0;
}

void Board::Draw(HDC dc)
{
    unsigned int i;

    for (i=0;i<All.size();i++)
        All[i]->Draw(dc);
    if (!Move->isEmpty())
        Move->Draw(dc);
}

void Board::Deal()
{
    //TODO: Add your source code here
}

void Board::InitGame()
{
    //TODO: Add your source code here
}

//********************************************************
// Translates screen position into a card stack
//********************************************************
StackPtr Board::MapMouse(int x, int y)
{
    unsigned int i;

    for (i=0;i<All.size();i++)
        if (All[i]->ContainsPoint(x,y))
            return All[i];
    return NULL;
}

#include "Main.h"

//********************************************************
// Starts a drag move
//********************************************************
bool Board::OnMouseDown(int button, int x, int y)
{
    StackPtr ptr;
    TRect r1,r2;
    RECT r;
    HWND hw=MainForm->Handle;

    if (button==0)
    {
        ptr=MapMouse(x,y);

        if (ptr.GetPtr()==NULL)
            return false;
        r1=ptr->GetRect();
        Move->SetPos(x,y);
        int n=ptr->GetSelectCount(x,y);
        Move->AddCards(ptr->RemoveCards(n));
        MoveSrc=ptr;
        r2=Move->GetRect();

        SetRect(&r,r1.Left,r1.Top,r1.Right,r1.Bottom);
        InvalidateRect(hw,&r,TRUE);
        SetRect(&r,r2.Left,r2.Top,r2.Right,r2.Bottom);
        InvalidateRect(hw,&r,TRUE);
    }

    return true;
}

//********************************************************
// Finishs a drag move and either moves the stack or
// returns it to its original location
//********************************************************
bool Board::OnMouseUp(int button, int x, int y)
{
    StackPtr ptr;
    TRect r1,r2,r3;
    RECT r;
    HWND hw=MainForm->Handle;

    if (Move->isEmpty())
            return false;
    if (button==0)
    {
        ptr=MapMouse(x,y);
        if (ptr.GetPtr()==NULL)
            ptr=MoveSrc;
        else if (!ValidateMove(ptr))
            ptr=MoveSrc;

        r2=Move->GetRect();
        int n=Move->GetNum();
        ptr->AddCards(Move->RemoveCards(n));
        if (MoveSrc!=ptr)
        {
            MoveSrc->UpdateStack();
            ptr->UpdateStack();
        }
        r1=ptr->GetRect();
        r3=MoveSrc->GetRect();

        SetRect(&r,r1.Left,r1.Top,r1.Right,r1.Bottom);
        InvalidateRect(hw,&r,TRUE);
        SetRect(&r,r2.Left,r2.Top,r2.Right,r2.Bottom);
        InvalidateRect(hw,&r,TRUE);
        SetRect(&r,r3.Left,r3.Top,r3.Right,r3.Bottom);
        InvalidateRect(hw,&r,TRUE);
    }
    return true;
}

//********************************************************
// Updates the screen location of the moving stack
//********************************************************
bool Board::OnMouseMove(int x, int y)
{
    TRect r1,r2;
    RECT r;
    HWND hw=MainForm->Handle;
    
    if (!Move->isEmpty())
    {
        r1=Move->GetRect();
        Move->SetPos(x,y);
        r2=Move->GetRect();

        SetRect(&r,r1.Left,r1.Top,r1.Right,r1.Bottom);
        InvalidateRect(hw,&r,TRUE);
        SetRect(&r,r2.Left,r2.Top,r2.Right,r2.Bottom);
        InvalidateRect(hw,&r,TRUE);
        return true;
    }
    return false;
}

//********************************************************
// Checks the proposed move against the loaded rules
//********************************************************
bool Board::ValidateMove(StackPtr dest)
{
    unsigned int i;
    int destarea,srcarea,j;

    // check that the source and destination are valid
    destarea=GetArea(dest);
    srcarea=GetArea(MoveSrc);
    for (i=0;i<moves.size();i++)
    {
        if (moves[i].dest!=AnyArea)
            if (moves[i].dest!=destarea)
                continue;
        if (moves[i].source!=AnyArea)
            if (moves[i].source!=srcarea)
                continue;
        CardPtr c1,c2;
        if (moves[i].move & SingleCard)
            if (Move->GetNum()!=1)
                continue;
        if ((moves[i].move & ValidStack)==ValidStack)
        {
            bool flag=false;;
            for (j=0;j<(Move->GetNum()-1);j++)
            {
                c1=Move->GetAt(j);
                c2=Move->GetAt(j+1);
                if (!ValidateCards(c1,c2,destarea))
                {
                    flag=true;
                    break;
                }
            }
            if (flag)
                continue;
        }

        c1=dest->GetTop();
        c2=Move->GetBottom();
        if (!ValidateCards(c1,c2,destarea))
            continue;
        // at this point the move is valid but requires the entire
        // stack to move so adjust the move stack to reflect this
        if (moves[i].move & FullStack)
        {
            int n=Move->GetNum();
            MoveSrc->AddCards(Move->RemoveCards(n));
            n=MoveSrc->GetNum();
            Move->AddCards(MoveSrc->RemoveCards(n));
            Move->ReverseOrder();
        }
        return true;
    }
    return false;
}

//********************************************************
// Determines which area a stack is in for purposes of
// rule application.
//********************************************************
int Board::GetArea(StackPtr ptr)
{
    if (ptr==Stock)
        return StockArea;
    if (ptr==Wastepile)
        return WastepileArea;
    if (ptr==Remove)
        return RemoveArea;
    unsigned int i;
    for (i=0;i<Tableau.size();i++)
        if (ptr==Tableau[i])
            return TableauArea;
    for (i=0;i<Foundation.size();i++)
        if (ptr==Foundation[i])
            return FoundationArea;
    for (i=0;i<Reserve.size();i++)
        if (ptr==Reserve[i])
            return ReserveArea;
    return 0;
}

//********************************************************
// Determines whether two cards can be stacked. Assumes
// that source and destination requirments have been met.
//********************************************************
bool Board::ValidateCards(CardPtr c1, CardPtr c2, int area)
{
    unsigned int i;
    MoveRule rule;

    // start by seting up a rule structure that describes
    // the propoesed move

    rule.dest=area;
    rule.rank=c2->GetRank();

    if (c1.GetPtr()==NULL)
    {
        rule.order=0;
        rule.move=OnBlank;
    }
    else
    {
        rule.move=0;
        if (c1->GetSuit()==c2->GetSuit())
            rule.order=SameSuit | SameColor;
        else if (c1->GetColor()==c2->GetColor())
            rule.order=SameColor;
        else
            rule.order=AltColor;

        if ((*c1)>(*c2))
        {
            if (c1->GetRank()==(c2->GetRank()+1))
                rule.order|=Descending;
        }
        else if ((*c1)<(*c2))
        {
            if (c1->GetRank()==(c2->GetRank()-1))
                rule.order|=Ascending;
        }
    }
    rule.order|=(Rank | AnySuit | AnyCard);

    // now check this move against the valid moves
    for (i=0;i<moves.size();i++)
    {
        // todo: allow moves which do not create a valid stack
        if (!moves[i].validate)
            continue;

        if (moves[i].move & OnBlank)
            if (rule.move!=OnBlank)
                continue;
        
        if (moves[i].dest!=AnyArea && moves[i].dest!=rule.dest)
            continue;
        if ((moves[i].order & rule.order)!=moves[i].order)
            continue;
        if ((moves[i].order & Rank)==Rank)
            if (moves[i].rank!=rule.rank)
                continue;
        return true;
    }
    return false;
}

//********************************************************
// Clears out the board to prepare for a new game.
//********************************************************
void Board::Clear()
{
    for (unsigned int i=0;i<All.size();i++)
        All[i]->Clear();
    Move->Clear();
    Tableau.clear();
    Foundation.clear();
    Reserve.clear();
    All.clear();
    
}

//********************************************************
// Determines if the victory condition has been met.
//********************************************************
bool Board::CheckFinish()
{
    unsigned int i,n1,n2;
    //StackPtr ptr;

    if (finish==TableauEmpty)
    {
        for (i=0;i<Tableau.size();i++)
            if (!Tableau[i]->isEmpty())
                return false;
        return true;
    }
    else if (finish==AllInFoundation)
    {
        n1=n2=0;
        for (i=0;i<All.size();i++)
            n1+=All[i]->GetNum();
        for (i=0;i<Foundation.size();i++)
            n2+=Foundation[i]->GetNum();
        return (n1==n2);
    }
    else if (finish==SingleStack)
    {
        n1=0;
        for (i=0;i<All.size();i++)
            if (All[i]->GetNum()>0)
                n1++;
        return (n1==1);
    }
    return false;
}