//---------------------------------------------------------------------------

#pragma hdrstop

#include "cards.h"
#include "CardsInterface.h"

// code in namespace Cards16DllInterface is from Matt Pietrek
// Class code is by Mark Kile

//---------------------------------------------------------------------------
#pragma package(smart_init)


namespace Cards16DllInterface
{

#ifdef __cplusplus
extern "C" {
#endif

#include <wownt32.h>

HINSTANCE	WINAPI LoadLibrary16( PSTR );
void		WINAPI FreeLibrary16( HINSTANCE );
FARPROC		WINAPI GetProcAddress16( HINSTANCE, PSTR );

#ifdef __cplusplus
}
#endif

static HINSTANCE	hLib16;
static FARPROC		QT_Thunk;

//
// don't need prototypes, these are being called from assembler
//
static FARPROC cdtInit16;
static FARPROC cdtDraw16;
static FARPROC cdtTerm16;
static FARPROC cdtDrawEx16;

static WORD  h16bitmem;
static DWORD p16bitmem;

static int *p32bitmem;

static WORD  retval;


//
// we need to use globals for the thunking functions
// to work correctly. (i.e. we can't pass parameters
// to a thunking function)
//
static WORD      hdc16bit;
static short int global_x, global_y;
static short int global_card, global_type;
static DWORD	 global_cardback;
static short int global_width, global_height;
static int		 global_cardwidth, global_cardheight;

#pragma optimize( "", off )

int CallcdtInitThunk()
{
	//put a dummy "scratch-pad" on the stack for QT_Thunk
	//need to make sure that the optimiser does not optimize
	//this away, so write to the first byte, and also
	//return a value from the array (see last statement of this function)
	unsigned char dummy[0x40];
	dummy[0] = 0;

	// allocate some memory in 16bit space!!!!!
	// This is necessary because cdtInit expects to be passed
	// the address of two integers. The 16bit function obviously
	// needs to be able to store values through these pointers, so
	// we need to allocate some memory in 16bit space.
	p16bitmem = WOWGlobalAllocLock16(GMEM_FIXED |GMEM_ZEROINIT| GMEM_SHARE, 32, &h16bitmem);

	//find the 32bit address
	p32bitmem = (int *)WOWGetVDMPointerFix(p16bitmem, 32, 1);

	//call cdtInit in 16bit code!!!
	__asm
	{
		mov		eax, [p16bitmem]
		push    eax
		add		ax, 4
		push    eax
		mov     edx, [cdtInit16]
		call    QT_Thunk
		mov     [retval], ax
	}

	// retrieve the width and height from the 16bit address space.
	global_cardwidth  = p32bitmem[0];
	global_cardheight = p32bitmem[1];

	//free the 16bit memory
	WOWGlobalUnlockFree16(p16bitmem);

	return retval;
}

WORD CallcdtDrawThunk()
{
	unsigned char dummy[0x40];
	dummy[0] = 0;

	__asm
	{
		xor		eax, eax
		mov		ax, [hdc16bit]
		push    ax			//hdc		(16bits)

		mov		ax, [global_x]
		push    ax			//x			(16bits)

		mov		ax, [global_y]
		push    ax			//y			(16bits)

		mov		ax, [global_card]
		push	ax			//cardnum	(16bits)

		mov		ax, [global_type]
		push	ax			//type		(16bits)

		mov		eax, [global_cardback]
		push	eax			//back		(32bits)

		mov     edx, [cdtDraw16]
		call    QT_Thunk
		mov     [retval], ax
	}

	return retval;
}

WORD CallcdtDrawExThunk()
{
	unsigned char dummy[0x40];
	dummy[0] = 0;

	__asm 
	{        
		xor		eax, eax
		mov		ax, [hdc16bit]
		push    ax			//hdc		(16bits)

		mov		ax, [global_x]
		push    ax			//x			(16bits)
		
		mov		ax, [global_y]
		push    ax			//y			(16bits)
		
		mov		ax, [global_width]
		push    ax			//x			(16bits)

		mov		ax, [global_height]
		push    ax			//y			(16bits)

		mov		ax, [global_card]
		push	ax			//cardnum	(16bits)
		
		mov		ax, [global_type]
		push	ax			//type		(16bits)
		
		mov		eax, [global_cardback]
		push	eax			//back		(32bits)
		
		mov     edx, [cdtDrawEx16]
		call    QT_Thunk
		mov     [retval], ax    
	}

	return retval;
}

#pragma optimize( "", on )

BOOL WINAPI cdtDrawExProxy(HDC hdc, int x, int y, int width, int height, int card, int type, DWORD back)
{
	hdc16bit = WOWHandle16(hdc, WOW_TYPE_HDC);

	global_x =		(short)x;
	global_y =		(short)y;
	global_width =	(short)width;
	global_height = (short)height;
	global_card =	(short)card;
	global_type =	(short)type;
	global_cardback=(short)back;

	return CallcdtDrawExThunk();
}

//
//	Need to implement the CallcdtAnimateThunk() function..
//
BOOL WINAPI cdtAnimateProxy(HDC hdc, int cardback, int x, int y, int frame)
{
	hdc16bit = WOWHandle16(hdc, WOW_TYPE_HDC);

	return FALSE;
}

#pragma optimize( "", off )

WORD CallcdtTermThunk(void)
{
	unsigned char dummy[0x40];
	dummy[0] = 0;

	__asm 
	{
		mov     edx, [cdtTerm16]
		call    QT_Thunk
		mov     [retval], ax
	}

	return dummy[1];
}

#pragma optimize( "", on )


} // namespace


//****************************************
//
// CardsInterface 16 bit version
// used for Win98 and previous
//
//****************************************

bool CardsInterface16::Init()
{
    using namespace Cards16DllInterface;

    HMODULE hKernel;		//Usual Windows API calls
	hKernel  = GetModuleHandle("KERNEL32");
	QT_Thunk = GetProcAddress(hKernel, "QT_Thunk");

	hLib16 = LoadLibrary16("cards.dll");


	if(hLib16 == 0)
	{
		return false;
	}

    cdtInit16 = (FARPROC)GetProcAddress16(hLib16, "cdtInit");
    cdtDraw16 = (FARPROC)GetProcAddress16(hLib16, "cdtDraw");
    cdtTerm16 = (FARPROC)GetProcAddress16(hLib16, "cdtTerm");

    CallcdtInitThunk();
	width = global_cardwidth;
	height = global_cardheight;

	return true;
}

bool CardsInterface16::Draw(HDC dc, int x, int y, int card, int type, DWORD color)
{
    using namespace Cards16DllInterface;
    hdc16bit = WOWHandle16(dc, WOW_TYPE_HDC);

	global_x =		(short)x;
	global_y =		(short)y;
	global_card =	(short)card;
	global_type =	(short)type;
	global_cardback=(short)color;

	return CallcdtDrawThunk();
}

bool CardsInterface16::DrawEx(HDC hdc, int x, int y, int dx, int dy, int card, int type, DWORD color)
{
    return false;
}

bool CardsInterface16::Animate(HDC hdc, int cardback, int x, int y, int frame)
{
    return false;
}

void CardsInterface16::Term()
{
    using namespace Cards16DllInterface;
    CallcdtTermThunk();
    FreeLibrary16(hLib16);
}

//****************************************
//
// CardsInterface 32 bit version
// used for WinNT and post Win98
//
//****************************************

bool CardsInterface32::Init()
{
    dll=LoadLibrary("cards.dll");
    if (dll==NULL)
        return false;
    cdtInit    = (pfcdtInit)    GetProcAddress(dll, "cdtInit");
	cdtDraw    = (pfcdtDraw)    GetProcAddress(dll, "cdtDraw");
	cdtDrawEx  = (pfcdtDrawEx)  GetProcAddress(dll, "cdtDrawExt");
	cdtAnimate = (pfcdtAnimate) GetProcAddress(dll, "cdtAnimate");
	cdtTerm    = (pfcdtTerm)    GetProcAddress(dll, "cdtTerm");

    cdtInit(&width,&height);
    return true;
}

bool CardsInterface32::Draw(HDC hdc, int x, int y, int card, int type, DWORD color)
{
    cdtDraw(hdc,x,y,card,type,color);
    return true;
}

bool CardsInterface32::DrawEx(HDC hdc, int x, int y, int dx, int dy, int card, int type, DWORD color)
{
    return false;
}

bool CardsInterface32::Animate(HDC hdc, int cardback, int x, int y, int frame)
{
    return false;
}

void CardsInterface32::Term()
{
    cdtTerm();
    FreeLibrary(dll);
}

//*****************************
// Generate the appropriate object for the abstract
// class CardsInterface
//*****************************

namespace CardsPtrWrapper
{
    CardsInterface *ptr=NULL;
}


CardsInterface* GetCardsInterface()
{
    if (CardsPtrWrapper::ptr==NULL)
    {
        if(GetVersion() < 0x80000000)
            CardsPtrWrapper::ptr=new CardsInterface32;
        else
            CardsPtrWrapper::ptr=new CardsInterface16;
    }
    return CardsPtrWrapper::ptr;
}
