//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "stdinclude.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

CardStack::CardStack(int x,int y)
{
    m_x=x;
    m_y=y;
    m_facedown=false;
}

CardStack::~CardStack()
{
    //TODO: Add your source code here
}

bool CardStack::AddCard(CardPtr c)
{
    m_list.push_back(c);
    return true;
}

bool CardStack::AddCards(vcards cards)
{
    for (unsigned int i=0;i<cards.size();i++)
    {
        if (m_facedown)
            cards[i]->SetFaceDown();
        m_list.push_back(cards[i]);
    }
    return true;
}

vcards CardStack::RemoveCards(unsigned int n)
{
    vcards v;

    if (n<=m_list.size())
    {
        for (unsigned int i=m_list.size()-n;i<m_list.size();i++)
            v.push_back(m_list[i]);
        for (unsigned int j=0;j<n;j++)
            m_list.pop_back();
    }
    return v;
}

bool CardStack::Contains(CardPtr c)
{
    for (unsigned int i=0;i<m_list.size();i++)
        if ((*c)==(*(m_list[i])))
            return true;
    return false;
}

CardPtr CardStack::GetTop()
{
    CardPtr c=0;
    unsigned int i=m_list.size();
    if (i>0)
        c=m_list[i-1];
    return c;
}

CardPtr CardStack::GetBottom()
{
    CardPtr c=0;
    unsigned int i=m_list.size();
    if (i>0)
        c=m_list[0];
    return c;
}

bool CardStack::isEmpty()
{
    return (m_list.size()==0);
}

bool CardStack::isFull()
{
    return false;
}

bool CardStack::ContainsPoint(int x, int y)
{
    TRect r=GetSize();
    if (x>=m_x && x<=(m_x+r.Right))
        if (y>=m_y && y<=(m_y+r.Bottom))
            return true;
    return false;
}

void CardStack::SetPos(int x, int y)
{
    m_x=x;
    m_y=y;
}

int CardStack::GetNum()
{
    return m_list.size();
}

void CardStack::DrawOutline(HDC dc)
{
    //TODO: Add your source code here
    int w,h;
    w=GetCardsInterface()->GetWidth();
    h=GetCardsInterface()->GetHeight();

    MoveToEx(dc,m_x,m_y,0);
    LineTo(dc,m_x+w,m_y);
    LineTo(dc,m_x+w,m_y+h);
    LineTo(dc,m_x,m_y+h);
    LineTo(dc,m_x,m_y);
}

CardPtr CardStack::GetAt(unsigned int pos)
{
    if (pos>m_list.size())
        return NULL;
    return m_list[pos];
}

void CardStack::ReverseOrder()
{
    //TODO: Add your source code here
    vcards tmp=RemoveCards(m_list.size());
    for (unsigned int i=tmp.size();i>0;i--)
        AddCard(tmp[i-1]);
}

void CardStack::Clear()
{
    m_list.clear();
}

TRect CardStack::GetRect()
{
    TRect r=GetSize();
    r=Rect(m_x,m_y,m_x+r.Right,m_y+r.Bottom);
    return r;
}

//*********************************************
// Drop Stack Specifics
//*********************************************

DropStack::DropStack(int x,int y)
    : CardStack(x,y)
{
    vspace=GetCardsInterface()->GetHeight()*0.25;
}

void DropStack::Draw(HDC dc)
{
    if (m_list.size()==0)
        DrawOutline(dc);
    for (unsigned int i=0;i<m_list.size();i++)
        m_list[i]->Draw(dc,m_x,m_y+vspace*i);
}

TRect DropStack::GetSize()
{
    int x,y;
    x=GetCardsInterface()->GetWidth();
    y=GetCardsInterface()->GetHeight()+(m_list.size()-1)*vspace;
    TRect r(0,0,x,y);
    return r;
}

void DropStack::UpdateStack()
{
    CardPtr c=GetTop();
    if (c.GetPtr()!=NULL)
        c->SetFaceUp();
}

int DropStack::GetSelectCount(int x, int y)
{
    TRect r=GetSize();
    int dist=r.Bottom-(y-m_y);
    dist-=GetCardsInterface()->GetHeight();
    int c=1;
    int n=m_list.size();
    while (dist>0)
    {
        dist-=vspace;
        c++;
        if (m_list[n-c]->GetFaceDown())
        {
            c--;
            return c;
        }
    }
    return c;
}

//*********************************************
// Top Stack Specifics
//*********************************************

TopStack::TopStack(int x,int y)
    : CardStack(x,y)
{
}

void TopStack::Draw(HDC dc)
{
    if (m_list.size()>0)
        m_list[m_list.size()-1]->Draw(dc,m_x,m_y);
    else
        DrawOutline(dc);
}

TRect TopStack::GetSize()
{
    int x,y;
    x=GetCardsInterface()->GetWidth();
    y=GetCardsInterface()->GetHeight();
    TRect r(0,0,x,y);
    return r;
}

void TopStack::UpdateStack()
{
    if (!m_facedown)
    {
        CardPtr c=GetTop();
        if (c.GetPtr()!=NULL)
            c->SetFaceUp();
    }
}

int TopStack::GetSelectCount(int x, int y)
{
    return 1;
}


